from httmock import all_requests, HTTMock, response
import pandas as pd
from pandas.util.testing import assert_frame_equal

from pyzoopla.for_sale import ForSaleSearch


def test_simple_search():

    @all_requests
    def zoopla_mock(url, request):
        content = open('tests/test_data/london.txt', 'r').read()
        return response(content=content, request=request)

    with HTTMock(zoopla_mock):
        results = ForSaleSearch(location='london')

    df_out = results.all_listings(page_limit=1).drop('date_generated', axis=1)
    df_out.listing_price = df_out.listing_price.astype('str')
    df_expected = pd.read_csv('tests/test_data/london_expected.csv', dtype={'listing_id': 'str'},
                              parse_dates=['listing_date'], keep_default_na=False)

    assert_frame_equal(df_out, df_expected)
    assert str(results) == 'https://www.zoopla.co.uk/for-sale/property/london/?include_sold=true&page_size=100&pn=1'
    assert results.slug == 'for-sale'
    assert results.assumed_search_location == 'London'
    assert results.total_properties == 10000
    assert results.total_pages == 100
    assert results.distance is None
    assert results.added is None
    assert results.include_sold


def test_complex_search():

    @all_requests
    def zoopla_mock(url, request):
        content = open('tests/test_data/sw7.txt', 'r').read()
        return response(content=content, request=request)

    with HTTMock(zoopla_mock):
        results = ForSaleSearch(location='sw7', new_build=True, distance=10, min_price=300000, max_price=1000000,
                                min_beds=2, max_beds=4, has_parking_garage=True, added=7)

    df_out = results.all_listings(page_limit=1).drop(['date_generated'], axis=1)
    df_out.listing_price = df_out.listing_price.astype('str')
    df_expected = pd.read_csv('tests/test_data/sw7_expected.csv', dtype={'listing_id': 'str', 'num_beds': 'str'},
                              parse_dates=['listing_date'], keep_default_na=False)

    assert_frame_equal(df_out, df_expected)
    assert str(results) == 'https://www.zoopla.co.uk/new-homes/property/sw7/?radius=10&price_min=300000&' \
                           'price_max=1000000&beds_min=2&beds_max=4&added=7_days&include_sold=true&page_size=100&' \
                           'pn=1&feature=has_parking_garage'
    assert results.slug == 'new-homes'
    assert results.assumed_search_location == 'SW7'
    assert results.total_properties == 99
    assert results.total_pages == 1
    assert results.new_build
    assert results.distance == 10
    assert results.min_price == 300000
    assert results.max_price == 1000000
    assert results.min_beds == 2
    assert results.max_beds == 4
    assert results.has_parking_garage
    assert results.added == 7
    assert results.buyer_incentive is None
    assert results.has_balcony_terrace is None



from httmock import all_requests, HTTMock, response
import numpy as np
import pandas as pd
from pandas.util.testing import assert_frame_equal

from pyzoopla.properties import PropertyDetails


def test_property_details_all_methods():

    @all_requests
    def zoopla_mock(url, request):
        content = open('tests/test_data/property1.txt', 'r').read()
        return response(content=content, request=request)

    with HTTMock(zoopla_mock):
        results = PropertyDetails(10647960)

    assert str(results) == 'Property details for 7 Anstey Brook Weston Turville Aylesbury HP22 5RT'
    assert repr(results) == 'https://ww2.zoopla.co.uk/property/10647960'
    assert results.listing_id == 10647960
    assert results.slug == 'property'
    assert results.details() == {'acorn_type': '8', 'activity': 'property_details', 'country_code': 'gb',
                                 'incode': '5RT', 'listing_id': '10647960', 'location': 'HP225RT', 'num_baths': '2',
                                 'num_beds': '4', 'outcode': 'HP22', 'page': '/property/details/', 'postal_area': 'HP',
                                 'price': '512922', 'price_estimate': '512922', 'price_last_sale': '510000',
                                 'price_temptme': 'null', 'property_type': 'semi_detached', 'rental_value': '1583',
                                 'section': 'home-values'}
    assert results.location() == {'is_approximate': False, 'latitude': 51.79233, 'longitude': -0.75069}
    assert not results.for_sale()
    assert results.property_value() == {'buy': {'value': 513000, 'lower_bound': 503000, 'upper_bound': 523000},
                                        'rent': {'value': 1600, 'lower_bound': 1400, 'upper_bound': 1800},
                                        'confidence': 99.5}
    assert_frame_equal(results.value_change(), pd.DataFrame([
        ['last sold (Oct 2017)', 509950, 3000, 0.6], ['1 month ago', 513500, 450, -0.1],
        ['3 months ago', 509950, 2850, 0.6], ['6 months ago', 510450, 2650, 0.5], ['1 year ago', 531050, 18200, -3.4],
        ['2 years ago', 496600, 16200, 3.3], ['3 years ago', 442250, 70850, 16.0],
        ['4 years ago', 412700, 100300, 24.3], ['5 years ago', 372300, 140600, 37.8]],
        columns=['period', 'value', 'value_change', 'perc_change']))
    assert results.sales_history(dataframe=False) == {
        'date': ['Oct 2017', 'Jul 2017', 'Mar 2014', 'Apr 2013', 'Mar 1997'],
        'listing_id': [np.nan, '44447984', np.nan, '28558762', np.nan],
        'price': [510000, 500000, 390000, 410000, 111500],
        'status': ['Sold', 'Listed for sale', 'Sold', 'Listed for sale', 'Sold']}


def test_property_details_for_sale():

    @all_requests
    def zoopla_mock(url, request):
        content = open('tests/test_data/property2.txt', 'r').read()
        return response(content=content, request=request)

    with HTTMock(zoopla_mock):
        results = PropertyDetails(32458972)

    assert results.for_sale() == '45395750'


def test_property_details_empty_data():

    @all_requests
    def zoopla_mock(url, request):
        content = open('tests/test_data/property3.txt', 'r').read()
        return response(content=content, request=request)

    with HTTMock(zoopla_mock):
        results = PropertyDetails(31866527)

    assert len(results.value_change()) == 0
    assert not results.for_sale()
    assert results.sales_history() == {'date': [], 'listing_id': [], 'price': [], 'status': []}
    assert results.property_value() == {'buy': {'lower_bound': np.nan, 'upper_bound': np.nan, 'value': np.nan},
                                        'confidence': np.nan,
                                        'rent': {'lower_bound': np.nan, 'upper_bound': np.nan, 'value': np.nan}}


def test_property_details_all_data():

    @all_requests
    def zoopla_mock(url, request):
        content = open('tests/test_data/property4.txt', 'r').read()
        return response(content=content, request=request)

    with HTTMock(zoopla_mock):
        results = PropertyDetails(10605934)

    df = results.all_data()
    assert df.drop('date_generated', axis=1).to_dict(orient='list') == {
        'acorn_type': ['19'],
        'activity': ['property_details'],
        'country_code': ['gb'],
        'incode': ['8DJ'],
        'listing_id': ['10605934'],
        'location': ['HP178DJ'],
        'num_baths': ['1'],
        'num_beds': ['2'],
        'outcode': ['HP17'],
        'page': ['/property/details/'],
        'postal_area': ['HP'],
        'price': ['329689'],
        'price_estimate': ['329689'],
        'price_last_sale': ['337000'],
        'price_temptme': ['null'],
        'property_type': ['terraced'],
        'rental_value': ['924'],
        'section': ['home-values'],
        'address': ['28 Anxey Way Haddenham Aylesbury HP17 8DJ'],
        'id': [10605934],
        'geolocation': [{'is_approximate': False, 'latitude': 51.772993, 'longitude': -0.933549}],
        'for_sale_id': [False],
        'property_value': [{'buy': {'value': 330000, 'lower_bound': 323000, 'upper_bound': 336000},
                            'rent': {'value': 900, 'lower_bound': 800, 'upper_bound': 1050},
                            'confidence': 99.5}],
        'value_change': [{'period': {0: 'last sold (Mar 2017)', 1: '1 month ago', 2: '3 months ago', 3: '6 months ago',
                                     4: '1 year ago', 5: '2 years ago', 6: '3 years ago', 7: '4 years ago',
                                     8: '5 years ago'},
                          'value': {0: 337100, 1: 330350, 2: 332000, 3: 337750, 4: 346250, 5: 325100, 6: 283250,
                                    7: 264000, 8: 243000},
                          'value_change': {0: 7000, 1: 450, 2: 2050, 3: 7850, 4: 16200, 5: 5000, 6: 46700, 7: 66000,
                                           8: 86900},
                          'perc_change': {0: -2.1, 1: -0.1, 2: -0.6, 3: -2.3, 4: -4.7, 5: 1.5, 6: 16.5, 7: 25.0,
                                          8: 35.8}}],
        'sales_history': [{'date': ['Mar 2017', 'Nov 2016', 'Feb 2012', 'Aug 2011', 'Feb 2005', 'May 1999'],
                           'status': ['Sold', 'Listed for sale', 'Sold', 'Listed for sale', 'Sold', 'Sold'],
                           'price': [337000, 345000, 220000, 225000, 178000, 83000],
                           'listing_id': [np.nan, '42187797', np.nan, '22249243', np.nan, np.nan]}]
    }



from httmock import all_requests, HTTMock, response
import pandas as pd
from pandas.util.testing import assert_frame_equal

from pyzoopla.to_rent import ToRentSearch


def test_simple_search():

    @all_requests
    def zoopla_mock(url, request):
        content = open('tests/test_data/london_rent.txt', 'r').read()
        return response(content=content, request=request)

    with HTTMock(zoopla_mock):
        results = ToRentSearch(location='london')

    df_out = results.all_listings(page_limit=1).drop('date_generated', axis=1)
    df_expected = pd.read_csv('tests/test_data/london_rent_expected.csv', dtype={'listing_id': 'str'},
                              parse_dates=['listing_date'], keep_default_na=False)

    assert_frame_equal(df_out, df_expected)
    assert str(results) == 'https://www.zoopla.co.uk/to-rent/property/london/?page_size=100&pn=1&' \
                           'price_frequency=per_month'
    assert results.slug == 'to-rent'
    assert results.assumed_search_location == 'London'
    assert results.total_properties == 10000
    assert results.total_pages == 100
    assert results.distance is None
    assert results.added is None


def test_complex_search():

    @all_requests
    def zoopla_mock(url, request):
        content = open('tests/test_data/sw7_rent.txt', 'r').read()
        return response(content=content, request=request)

    with HTTMock(zoopla_mock):
        results = ToRentSearch(location='sw7', available=3, distance=10, min_price=100, max_price=2000,
                               min_beds=2, max_beds=4, has_parking_garage=True, added=7)

    df_out = results.all_listings(page_limit=1).drop(['date_generated'], axis=1)
    df_out.listing_price = df_out.listing_price.astype('str')
    df_expected = pd.read_csv('tests/test_data/sw7_rent_expected.csv', dtype={'listing_id': 'str', 'num_beds': 'str'},
                              parse_dates=['listing_date'], keep_default_na=False)

    assert_frame_equal(df_out, df_expected)
    assert str(results) == 'https://www.zoopla.co.uk/to-rent/property/sw7/?radius=10&price_min=100&' \
                           'price_max=2000&beds_min=2&beds_max=4&added=7_days&page_size=100&pn=1&' \
                           'available_from=3months&price_frequency=per_month&feature=has_parking_garage'
    assert results.slug == 'to-rent'
    assert results.assumed_search_location == 'SW7'
    assert results.total_properties == 1021
    assert results.total_pages == 11
    assert results.distance == 10
    assert results.min_price == 100
    assert results.max_price == 2000
    assert results.min_beds == 2
    assert results.max_beds == 4
    assert results.has_parking_garage
    assert results.added == 7
    assert results.has_balcony_terrace is None
    assert results.furnished_state is None



import logging
import pymysql
import sys

from pyzoopla.utils import insert_into_db

logging.basicConfig(level=logging.INFO, format='%(asctime)-15s %(funcName)s:%(lineno)d %(levelname)s - %(msg)s')


class BaseScraper:
    def __init__(self, location):
        self.location = location

    def search_prices(self):
        """Return search results object."""
        raise NotImplementedError

    def save_data(self, database=None, port=None, user=None, password=None):
        """Output data to sql database."""
        raise NotImplementedError


class BasePurchaseScraper:
    def __init__(self, location, distance, scraper_object):
        self.location = location
        self.distance = distance
        self.scraper_object = scraper_object

    def search_prices(self):
        results = self.scraper_object(location=self.location, distance=self.distance)

        try:
            _ = results.assumed_search_location
        except AttributeError:
            logging.critical('No location named \'{}\''.format(self.location))
            sys.exit()

        logging.info('Found {:,} properties {} in {} pages for location {} with radius {} miles.'.format(
            results.total_properties, results.slug.replace('-', ' '), results.total_pages,
            results.assumed_search_location, results.distance))

        if results.total_properties > 10000:
            logging.warning('A maximum of 10,000 properties can be scraped for this location.')

        return results, results.assumed_search_location, results.total_pages

    def save_data(self, database=None, port=None, user=None, password=None):

        db_conn = pymysql.connect(host=database, port=port, user=user, password=password, charset='utf8',
                                  cursorclass=pymysql.cursors.DictCursor)
        logging.info('Connected to database host: {}'.format(database))

        results, search_location, total_pages = self.search_prices()

        for page in range(1, total_pages + 1):
            df = results.all_listings_page(page)
            for row in df.to_dict(orient='index').values():
                try:
                    logging.info('Scraping details for listing ID: {}'.format(row['listing_id']))
                    insert_into_db(db_conn=db_conn, data=row, schema='zdb',
                                   table='{}_listings'.format(results.slug.replace('-', '_')))
                except AttributeError as err:
                    logging.info('Could not scrape listing ID {}: {}'.format(row['listing_id'], err))
                    continue

        db_conn.close()
        logging.info('Finished scraping {} listing details for location {}.'.format(results.slug, search_location))



from bs4 import BeautifulSoup
import pytest

from tests.test_utils_data import *
from pyzoopla.utils import *


@pytest.fixture(scope='module')
def station_html():
    return [BeautifulSoup(html1, 'lxml'), BeautifulSoup(html2, 'lxml')]


def test_currency_to_num():
    assert currency_to_num('£100,000') == 100000
    assert currency_to_num('£100,000,000') == 100000000
    assert currency_to_num('100,000,000') == 100000000
    assert currency_to_num('£100') == 100
    assert currency_to_num('100') == 100
    assert currency_to_num(' \n £100,000 \n ') == 100000
    assert currency_to_num('£100.30', data_type=float) == 100.3
    assert currency_to_num('£100.30', data_type=int) == '100.30'
    assert currency_to_num(' £1,123,456.99\n pcm ', data_type=float) == 1123456.99


def test_myround():
    assert myround(110380, base=50) == 110400
    assert myround(110375, base=50) == 110400
    assert myround(110374, base=50) == 110350
    assert myround(110374.99, base=50) == 110350
    assert myround(110375.0, base=50) == 110400
    assert myround(10, base=10) == 10
    assert myround(0, base=10) == 0
    assert myround(0, base=100) == 0
    assert myround(210, base=10) == 210
    assert myround(215, base=10) == 220
    assert myround(214, base=10) == 210
    assert myround(214.99, base=10) == 210


@pytest.mark.parametrize('html,expected,data_type', [
    ('<span class="price-modifier">Guide price</span>', 'Guide price', str),
    ('<span class="num-icon num-beds" title="3 bedrooms"><span class="interface"></span>3</span>', 3, int),
    ('<span class="num-icon num-baths" title="1 bathroom"><span class="interface"></span>1</span>', 1, int),
    ('<span class="num-icon num-reception" title="2 reception rooms"><span class="interface"></span>2</span>', 2, int),
    ('<span class="num-icon num-beds" title="3 bedrooms"></span>', '', str)
])
def test_text_or_none(html, expected, data_type):
    assert text_or_none(soup=BeautifulSoup(html, 'lxml'), data_type=data_type) == expected


def test_to_datetime():
    assert to_datetime('1st Jun 2010') == datetime(2010, 6, 1, 0, 0)
    assert to_datetime('2nd Apr 2010') == datetime(2010, 4, 2, 0, 0)
    assert to_datetime('3rd Mar 2010') == datetime(2010, 3, 3, 0, 0)
    assert to_datetime('4th Feb 2009') == datetime(2009, 2, 4, 0, 0)
    assert to_datetime('19th Jan 2010') == datetime(2010, 1, 19, 0, 0)
    assert to_datetime('21st May 2012') == datetime(2012, 5, 21, 0, 0)


def test_get_station_name():
    data = station_html()
    assert get_station_name(soup=data, station_num=0) == 'London Bridge'
    assert get_station_name(soup=data, station_num=1) == 'Borough'


def test_dist_to_num():
    data = station_html()
    assert dist_to_num(data[0]) == 0.4
    assert dist_to_num(data[1]) == 1.2


def test_text_inbetween():
    string = 'The quick brown fox jumps over the lazy dog'
    assert text_inbetween(text=string, left='brown ', right=' over') == 'fox jumps'
    assert text_inbetween(text=string, left='brown', right='over') == ' fox jumps '
    assert text_inbetween(text=string, left='T', right='k') == 'he quic'
    assert text_inbetween(text=string, left='brown', right='fox') == ' '



from httmock import all_requests, HTTMock, response
import pytest

from pyzoopla.prices import PricesSearch


def test_simple_prices_search():

    @all_requests
    def zoopla_mock(url, request):
        content = open('tests/test_data/w1.txt', 'r').read()
        return response(content=content, request=request)

    with HTTMock(zoopla_mock):
        results = PricesSearch('w1')

    assert str(results) == 'https://www.zoopla.co.uk/house-prices/w1/'
    assert results.location == 'w1'
    assert results.assumed_search_location == 'W1'
    assert results.total_properties == 18057
    assert results.total_pages == 452
    assert results.num_months is None
    assert results.property_type_code is None
    assert results.market_activity() == {'period': 20, 'property_type': 'all', 'average_price': 815521,
                                         'num_sales': 11192, 'average_value': 2041815, 'value_change': 1594930}
    assert results.market_activity(period=1) == {'period': 1, 'property_type': 'all', 'average_price': 2101974,
                                                 'num_sales': 243, 'average_value': 2041815, 'value_change': 57639}
    assert results.market_activity(period=5, property_type='terraced') == {'period': 5, 'property_type': 'terraced',
                                                                           'average_price': 4113439, 'num_sales': 132,
                                                                           'average_value': 4201518,
                                                                           'value_change': 765831}
    assert results.all_properties(page_end=1) == ['24984791', '24984824', '24984827', '32015918', '31224282',
                                                  '31966815', '24984831', '24984833', '29168144', '24984819',
                                                  '32352479', '24984792', '29467841', '28263418', '32667778',
                                                  '32441152', '28833087', '24984784', '24984785', '24984786',
                                                  '24984787', '24984768', '24984769', '29023006', '31165661',
                                                  '24984645', '24984817', '24984832', '31964512', '29876454',
                                                  '29019891', '31464359', '32622368', '32695776', '24984834',
                                                  '29061498', '29876455', '31299781', '31299782', '31299783']


def test_complex_prices_search():

    @all_requests
    def zoopla_mock(url, request):
        content = open('tests/test_data/e1.txt', 'r').read()
        return response(content=content, request=request)

    with HTTMock(zoopla_mock):
        results = PricesSearch('e1', num_months=3, property_type_code='F')

    assert str(results) == 'https://www.zoopla.co.uk/house-prices/e1/?num_months=3&property_type_code=F'
    assert results.location == 'e1'
    assert results.assumed_search_location == 'E1'
    assert results.total_properties == 38
    assert results.total_pages == 1
    assert results.num_months == 3
    assert results.property_type_code == 'F'
    assert results.market_activity() == {'period': 20, 'property_type': 'all', 'average_price': 326240,
                                         'num_sales': 21206, 'average_value': 595808, 'value_change': 458353}
    assert results.market_activity(period=1) == {'period': 1, 'property_type': 'all', 'average_price': 585793,
                                                 'num_sales': 349, 'average_value': 595808, 'value_change': 20715}
    assert results.market_activity(period=5, property_type='terraced') == {'period': 5, 'property_type': 'terraced',
                                                                           'average_price': 861817, 'num_sales': 248,
                                                                           'average_value': 792981,
                                                                           'value_change': 224826}
    with pytest.raises(ValueError):
        results.market_activity(period=2)
    with pytest.raises(ValueError):
        results.market_activity(property_type='bungalow')
    assert results.all_properties(page_end=1) == ['29594665', '7605668', '7605776', '7608535', '29503014', '7627919',
                                                  '30904195', '7609811', '30973201', '7607566', '7620679', '7625359',
                                                  '7614326', '7626841', '7604715', '7619868', '7629406', '7622162',
                                                  '7618631', '7622942', '7605936', '7631711', '7635734', '7635298',
                                                  '7632047', '7630842', '7633108', '7630918', '7634609', '7631034',
                                                  '7634653', '7631426', '7629831', '7632658', '7635546', '7631329',
                                                  '7635637', '7633019']



from httmock import all_requests, HTTMock, response

from pyzoopla.listing import PropertyHistoricalListing, PropertyListing


def test_listing_few_details():

    @all_requests
    def zoopla_mock(url, request):
        content = open('tests/test_data/listing.txt', 'r').read()
        return response(content=content, request=request)

    with HTTMock(zoopla_mock):
        results = PropertyListing(47902463)

    assert str(results) == 'https://ww2.zoopla.co.uk/for-sale/details/47902463'
    assert results.listing_id == 47902463
    assert results.slug == 'for-sale/details'

    data = results.details(dataframe=False)
    del data['date_generated']
    assert data == {
        'listing_id': 47902463,
        'description': "\n                        A lovely three, three bathroom bedroom third floor Marylebone "
                       "apartment in a prestigious mansion block with lift and porter. Beautifully presented "
                       "throughout, comprising a master bedroom with en suite shower room, two further double "
                       "bedrooms, two further shower rooms, and a large semi open plan kitchen/reception room with "
                       "dining area. Further features bright and charming rooms, neutral décor and ample storage.You "
                       "may download, store and use the material for your own personal use and research. You may not "
                       "republish, retransmit, redistribute or otherwise make the material available to any party or "
                       "make the same available on any website, online service or bulletin board of your own or of "
                       "any other party or make the same available in hard copy or in any other media without the "
                       "website owner's express prior written consent. The website owner's copyright must remain on "
                       "all reproductions of material taken from this website.\n                    ",
        'main_features': ['3                bedrooms', '3                bathrooms', '1                reception room',
                          'floor area1,163 sq. ft'],
        'more_features': [],
        'price_history': {'date': ['7th Jun 2018'],
                          'price': [2295000],
                          'detail': ['First listed']}
    }


def test_listing_more_details():

    @all_requests
    def zoopla_mock(url, request):
        content = open('tests/test_data/listing2.txt', 'r').read()
        return response(content=content, request=request)

    with HTTMock(zoopla_mock):
        results = PropertyListing(38834402)

    assert str(results) == 'https://ww2.zoopla.co.uk/for-sale/details/38834402'
    assert results.listing_id == 38834402
    assert results.slug == 'for-sale/details'

    data = results.details(dataframe=True).to_dict()
    del data['date_generated']
    assert data == {
        'listing_id': {0: 38834402},
        'description': {0: '\n                        Set within a superb portered building just south of Oxford '
                           'Street, this fantastic two bedroom, two bathroom apartment offers beautifully presented '
                           'living space with classic décor.A wealth of exclusive boutiques and eateries can be found '
                           'throughout Mayfair, Oxford Street and Regent Street offer world class shops and department'
                           ' stores. Hyde Park is also moments away.\n                    '},
        'main_features': {0: ['2                bedrooms']},
        'more_features': {0: ['Secure entry and lift access to the second floor',
                              'Generous reception room with lots of natural light',
                              'Separate modern kitchen with ample storage space',
                              'Master bedroom with fitted wardrobe and en suite',
                              'Good-sized second bedroom with fitted wardrobe',
                              'Well presented shower room',
                              'Large entrance hall with storage cupboards']},
        'price_history': {0: {'date': ['18th Apr 2018', '22nd Dec 2015', '19th Feb 2015', '29th Oct 2014'],
                              'price': [2300000, 2599000, 2800000, 3000000],
                              'detail': ['Price reduced by £299,000', 'Price reduced by £201,000',
                                         'Price reduced by £200,000', 'First listed']}}
    }


def test_historical_listing_details():

    @all_requests
    def zoopla_mock(url, request):
        content = open('tests/test_data/historical.txt', 'r').read()
        return response(content=content, request=request)

    with HTTMock(zoopla_mock):
        results = PropertyHistoricalListing(37047136)

    assert str(results) == 'Property history of 108 Shoreditch High Street, London E1 6JN, \n29th May 2015'
    assert results.listing_id == 37047136
    assert results.slug == 'property-history'

    data = results.details(dataframe=False)
    del data['date_generated']
    assert data == {
        'listing_id': 37047136,
        'description': ". . .                 This wonderfully bright and spacious one bedroom apartment occupies the "
                       "third floor of a sympathetically restored Victorian building.Offering approximately 734 sq. "
                       "Ft. Of space this larger than average one bedroom boasts a stylish finish in the form of "
                       "exposed brick work, wood flooring, double glazed sash windows and a bespoke kitchen with "
                       "Siemens ovens and induction hob.Comprising an open plan dual aspect kitchen and living space, "
                       "generous double bedroom with fitted wardrobes and a high quality bathroom with a large "
                       "walk-in shower and storage spaces.Enjoying a fantastic locationin the heart of vibrant "
                       "Shoreditch, home to an increasing number of boutique clothing shops, the Ace Hotel and an "
                       "array of excellent bars and restaurants. Fashionable Brick Lane and Columbia Road are also "
                       "close by.A number of transport links serve the property including Shoreditch High Street "
                       "(Overground) just a stone's throw away, Old Street Station (National Rail, Northern Line) and "
                       "the major hub of Liverpool Street.Offered with no onward chain.. .             . . . . ",
        'features': '. . 734 sq. Ft. One bedroom apartment. Victorian conversion. High specification finish. Exposed '
                    'brick/sash windows/wood flooring. Central Shoreditch location. . . . '
    }



from bs4 import BeautifulSoup
from datetime import datetime
import pandas as pd
import requests

from pyzoopla.utils import currency_to_num, dist_to_num, get_station_name, headers, text_or_none, to_datetime


class Base:
    """
    Base class which holds common methods for all classes in this package.
    """

    def __init__(self):
        self.html = self._get_html()
        self.soup = self._get_soup()

    def __repr__(self):
        """Default string representation of all package objects is its URL."""
        return self.html.url

    def __str__(self):
        return self.__repr__()

    def _get_html(self, page_no=None):
        """Makes request to website after constructing the relevant URL."""
        raise NotImplementedError

    def _get_soup(self, page_no=1):
        """Creates soup object from html."""
        return BeautifulSoup(self._get_html(page_no).text, 'lxml')

    def save(self, file_location):
        """Writes raw html to file."""
        with open(file_location, mode='wb') as file:
            file.write(self.html.content)


class BaseProperty(Base):
    """
    Base class for Zoopla pages containing the details of a single property.
    """

    def __init__(self, listing_id, slug):
        self.listing_id = listing_id
        self.slug = slug
        super(BaseProperty, self).__init__()

    def _get_html(self, page_no=None):
        payload = {'pn': page_no}
        url = 'https://ww2.zoopla.co.uk/{}/{}'.format(self.slug, self.listing_id)
        return requests.get(url, params=payload, headers=headers)


class BaseSearch(Base):
    """
    Base class for Zoopla pages containing the details of multiple properties i.e. search results.
    """

    def __init__(self, location, page_size=None):
        self.location = location.replace(' ', '-')
        self.page_size = page_size
        super(BaseSearch, self).__init__()

    @property
    def assumed_search_location(self):
        """Zoopla's assumed search string given user's input."""
        raise NotImplementedError

    @property
    def total_properties(self):
        """Total number of properties matching search criteria."""
        results = int(self.soup.find(name='span', attrs={'class': 'listing-results-utils-count'})
                      .text.split(' of ')[-1].replace(',', '').replace('+', ''))
        return results

    @property
    def total_pages(self):
        """Total number of pages of properties matching search criteria."""
        results = self.total_properties
        pages = (results // self.page_size) + 1 if results % self.page_size != 0 else results // self.page_size
        return pages


class BasePurchaseSearch(BaseSearch):
    """Base class for all property search results available to purchase (buy/rent)."""

    @property
    def assumed_search_location(self):
        return self.soup.find(name='title').text.split(' in ')[-1].split(' - ')[0]

    def all_listings_page(self, page_no):
        """Search result details for a single page"""

        soup = self.soup if page_no == 1 else self._get_soup(page_no)
        listings = soup.find_all(name='li', attrs={'class': 'srp clearfix '}) + \
            soup.find_all(name='li', attrs={'class': 'srp clearfix premium-listing premium-listing--branded '})

        df = pd.DataFrame()
        df['listing_id'] = [i['data-listing-id'] for i in listings]
        df['listing_price'] = [currency_to_num(
            i.find(name='a',
                   attrs={'class': 'listing-results-price text-price'}).text.strip().split('\n')[0]) for i in listings]
        df['price_modifier'] = [text_or_none(i.find(name='span', attrs={'class': 'price-modifier'})) for i in listings]
        df['address'] = [i.find(name='a', attrs={'class': 'listing-results-address'}).text for i in listings]
        df['summary'] = [i.find(name='a', attrs={'style': 'text-decoration:underline;'}).text for i in listings]
        df['num_beds'] = [text_or_none(i.find(name='span', attrs={'class': 'num-icon num-beds'}), data_type=str)
                          for i in listings]
        df['num_baths'] = [text_or_none(i.find(name='span', attrs={'class': 'num-icon num-baths'}), data_type=str)
                           for i in listings]
        df['num_receptions'] = [text_or_none(
            i.find(name='span', attrs={'class': 'num-icon num-reception'}), data_type=str) for i in listings]
        df['description'] = [i.find(name='p').text.strip() for i in listings]
        df['listing_date'] = [to_datetime(
            i.find_all(name='small')[-1].text.split('Listed on \n')[1].split('\n')[0].strip()) for i in listings]
        df['estate_agent'] = [i.find(name='p', attrs={'class': 'top-half listing-results-marketed'}).span.text
                              for i in listings]
        df['station1'] = [get_station_name(i.find_all('li', attrs={'class': 'clearfix'}), 0) for i in listings]
        df['distance1'] = [dist_to_num(i.find_all('li', attrs={'class': 'clearfix'})[0]) for i in listings]
        df['station2'] = [get_station_name(i.find_all('li', attrs={'class': 'clearfix'}), 1) for i in listings]
        df['distance2'] = [dist_to_num(i.find_all('li', attrs={'class': 'clearfix'})[1]) for i in listings]
        df['date_generated'] = datetime.utcnow()

        return df

    def all_listings(self, page_limit=None):
        """
        Summary of search results for all pages.
        :param page_limit: number of search result pages to consider.
        :return: Pandas dataframe.
        """

        page_limit = self.total_pages if not page_limit else page_limit
        data = [self.all_listings_page(page) for page in range(1, page_limit + 1)]
        return pd.concat(data).reset_index(drop=True)



import requests

from pyzoopla.base import BasePurchaseSearch
from pyzoopla.utils import headers


class ForSaleSearch(BasePurchaseSearch):
    """
    Search for properties for sale from https://www.zoopla.co.uk/for-sale/property/{location}

    distance: {0, 0.25, 0.5, 1, 3, 5, 10, 15, 20, 30, 40}
    property_type: {property, houses, flats, farms-land}
    results_sort: {newest_listings, highest_price, lowest_price, most_reduced, most_popular}
    """

    slug = 'for-sale'

    def __init__(self, location, page_size=100, distance=None, min_price=None, max_price=None, min_beds=None,
                 max_beds=None, property_type='property', added=None, results_sort=None, include_sold=True,
                 has_parking_garage=None, has_garden=None, has_fireplace=None, has_wood_floors=None,
                 has_balcony_terrace=None, is_rural_secluded=None, has_porter_security=None, chain_free=False,
                 reduced_price_only=False, new_build=None, is_shared_ownership=False, is_auction=False,
                 is_retirement_home=False, buyer_incentive=None, keywords=None):
        self.distance = distance
        self.min_price = min_price
        self.max_price = max_price
        self.property_type = property_type
        self.min_beds = min_beds
        self.max_beds = max_beds
        self.added = added
        self.results_sort = results_sort
        self.include_sold = include_sold
        self.has_parking_garage = 'has_parking_garage' if has_parking_garage else None
        self.has_garden = 'has_garden' if has_garden else None
        self.has_fireplace = 'has_fireplace' if has_fireplace else None
        self.has_wood_floors = 'has_wood_floors' if has_wood_floors else None
        self.has_balcony_terrace = 'has_balcony_terrace' if has_balcony_terrace else None
        self.is_rural_secluded = 'is_rural_secluded' if is_rural_secluded else None
        self.has_porter_security = 'has_porter_security' if has_porter_security else None
        self.chain_free = str(chain_free).lower() if chain_free else None
        self.reduced_price_only = str(reduced_price_only).lower() if reduced_price_only else None
        self.new_build = new_build
        self.is_shared_ownership = str(is_shared_ownership).lower() if is_shared_ownership else None
        self.is_auction = str(is_auction).lower() if is_auction else None
        self.is_retirement_home = str(is_retirement_home).lower() if is_retirement_home else None
        self.buyer_incentive = 'help_to_buy' if buyer_incentive else None
        self.keywords = '"{}"'.format(keywords) if keywords else None
        super(ForSaleSearch, self).__init__(location, page_size)

    def _get_html(self, page_no=1):

        if self.added == 1:
            added = '24_hours'
        elif self.added:
            added = '{}_days'.format(self.added)
        else:
            added = None

        new_homes = None
        if self.new_build is None:
            pass
        elif self.new_build:
            self.slug = 'new-homes'
        else:
            new_homes = 'exclude'

        payload = {'radius': self.distance, 'price_min': self.min_price, 'price_max': self.max_price,
                   'beds_min': self.min_beds, 'beds_max': self.max_beds, 'added': added,
                   'results_sort': self.results_sort, 'include_sold': str(self.include_sold).lower(),
                   'page_size': self.page_size, 'pn': page_no,
                   'feature': [self.has_parking_garage, self.has_garden, self.has_fireplace, self.has_wood_floors,
                               self.has_balcony_terrace, self.is_rural_secluded, self.has_porter_security],
                   'chain_free': self.chain_free, 'reduced_price_only': self.reduced_price_only,
                   'new_homes': new_homes, 'is_shared_ownership': self.is_shared_ownership,
                   'is_auction': self.is_auction, 'is_retirement_home': self.is_retirement_home,
                   'buyer_incentive': self.buyer_incentive, 'keywords': self.keywords}
        url = 'https://www.zoopla.co.uk/{}/{}/{}/'.format(self.slug, self.property_type, self.location)

        return requests.get(url, params=payload, headers=headers)



import argparse
import sys

from .scrape import ForSaleScraper, PropertyScraper, ToRentScraper


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument('--location', '-l', dest='loc', type=str, required=True, help='Name of location e.g. postcode')
    parser.add_argument('--database', '-d', dest='database', help='Name of database endpoint')
    parser.add_argument('--port', '-o', dest='port', default=3306, help='Database port number')
    parser.add_argument('--user', '-u', dest='user', help='Database username')
    parser.add_argument('--password', '-p', dest='password', help='Database password')
    parser.add_argument('--test', '-t', dest='test', action='store_true', help='Get no. of properties for location')
    parser.add_argument('--property', '-r', dest='property', action='store_true', help='Scrape property results')
    parser.add_argument('--sale', '-s', dest='sale', action='store_true', help='Scrape for-sale search results')
    parser.add_argument('--rent', '-n', dest='rent', action='store_true', help='Scrape to-rent search results')
    parser.add_argument('--distance', '-c', dest='distance', default=10, help='Radius around location to search')
    args = parser.parse_args()

    if args.test:
        properties = PropertyScraper(location=args.loc)
        _ = properties.search_prices()

        for_sale_results = ForSaleScraper(location=args.loc, distance=args.distance)
        _ = for_sale_results.search_prices()

        to_rent_results = ToRentScraper(location=args.loc, distance=args.distance)
        _ = to_rent_results.search_prices()

        sys.exit()

    if args.property:
        properties = PropertyScraper(location=args.loc)
        properties.save_data(database=args.database, port=args.port, user=args.user, password=args.password)

    if args.sale:
        search = ForSaleScraper(location=args.loc, distance=args.distance)
        search.save_data(database=args.database, port=args.port, user=args.user, password=args.password)

    if args.rent:
        search = ToRentScraper(location=args.loc, distance=args.distance)
        search.save_data(database=args.database, port=args.port, user=args.user, password=args.password)


if __name__ == '__main__':
    main()



import requests

from pyzoopla.base import BasePurchaseSearch
from pyzoopla.utils import headers


class ToRentSearch(BasePurchaseSearch):
    """
    Search for properties to rent from https://www.zoopla.co.uk/to-rent/property/{location}

    distance: {0, 0.25, 0.5, 1, 3, 5, 10, 15, 20, 30, 40}
    property_type: {property, houses, flats, farms-land}
    results_sort: {newest_listings, highest_price, lowest_price, most_reduced, most_popular}
    price_frequency: {per_month, per_week}
    rental_term: {short}
    furnished_state: {furnished, part_furnished, unfurnished}
    """

    slug = 'to-rent'

    def __init__(self, location, page_size=100, added=None, available=None, distance=None,
                 include_rented=None, include_shared_accommodation=None, price_frequency='per_month', min_price=None,
                 max_price=None, min_beds=None, max_beds=None, results_sort=None, has_parking_garage=None,
                 has_garden=None, has_fireplace=None, has_wood_floors=None, has_balcony_terrace=None,
                 is_rural_secluded=None, has_porter_security=None, property_type='property', rental_term=None,
                 pets_allowed=None, bills_included=None, furnished_state=None, keywords=None):
        self.added = added
        self.available = available
        self.distance = distance
        self.include_rented = str(include_rented).lower() if include_rented else None
        self.include_shared = str(include_shared_accommodation).lower() if include_shared_accommodation else None
        self.price_frequency = price_frequency
        self.min_price = min_price
        self.max_price = max_price
        self.min_beds = min_beds
        self.max_beds = max_beds
        self.results_sort = results_sort
        self.has_parking_garage = 'has_parking_garage' if has_parking_garage else None
        self.has_garden = 'has_garden' if has_garden else None
        self.has_fireplace = 'has_fireplace' if has_fireplace else None
        self.has_wood_floors = 'has_wood_floors' if has_wood_floors else None
        self.has_balcony_terrace = 'has_balcony_terrace' if has_balcony_terrace else None
        self.is_rural_secluded = 'is_rural_secluded' if is_rural_secluded else None
        self.has_porter_security = 'has_porter_security' if has_porter_security else None
        self.property_type = property_type
        self.rental_term = rental_term
        self.pets_allowed = str(pets_allowed).lower() if pets_allowed else None
        self.bills_included = str(bills_included).lower() if bills_included else None
        self.furnished_state = furnished_state
        self.keywords = '"{}"'.format(keywords) if keywords else None
        super(ToRentSearch, self).__init__(location, page_size)

    def _get_html(self, page_no=1):

        if self.added == 1:
            added = '24_hours'
        elif self.added:
            added = '{}_days'.format(self.added)
        else:
            added = None

        if self.available == 0:
            available = 'now'
        elif self.available:
            available = '{}months'.format(self.available)
        else:
            available = None

        payload = {'radius': self.distance, 'price_min': self.min_price, 'price_max': self.max_price,
                   'beds_min': self.min_beds, 'beds_max': self.max_beds, 'added': added,
                   'results_sort': self.results_sort, 'page_size': self.page_size, 'pn': page_no,
                   'include_rented': self.include_rented, 'include_shared_accommodation': self.include_shared,
                   'available_from': available, 'price_frequency': self.price_frequency,
                   'rental_term': self.rental_term, 'pets_allowed': self.pets_allowed,
                   'bills_included': self.bills_included, 'furnished_state': self.furnished_state,
                   'feature': [self.has_parking_garage, self.has_garden, self.has_fireplace, self.has_wood_floors,
                               self.has_balcony_terrace, self.is_rural_secluded, self.has_porter_security],
                   'keywords': self.keywords}
        url = 'https://www.zoopla.co.uk/{}/{}/{}/'.format(self.slug, self.property_type, self.location)

        return requests.get(url, params=payload, headers=headers)



import ast
from datetime import datetime
from itertools import chain
import numpy as np
import pandas as pd

from pyzoopla.base import BaseProperty
from pyzoopla.utils import currency_to_num, myround, text_inbetween


class PropertyDetails(BaseProperty):
    """
    Details of a property scraped from https://ww2.zoopla.co.uk/property/{property_id}
    """

    def __init__(self, property_id):
        super(PropertyDetails, self).__init__(property_id, slug='property')

    def __str__(self):
        # Address of the property
        return self.soup.find(name='title').text.split(' - ')[0]

    def details(self):
        string = text_inbetween(text=self.html.text.replace('\n', '').replace(' ', ''),
                                left='ZPG.trackData.taxonomy=', right=';</script><script>d')
        string = string.replace('{', '{"').replace(':', '":').replace(',', ',"').replace('null', '"null"')
        return ast.literal_eval(string)

    def location(self):
        string = text_inbetween(text=self.html.text.replace('\n', '').replace(' ', ''),
                                left='"coordinates":', right=',"pin":"https://r.zoocdn.com/assets/map-pin.png')
        string = string.replace('false', 'False').replace('true', 'True')
        return ast.literal_eval(string)

    def for_sale(self):
        # current listings available at https://www.zoopla.co.uk/for-sale/details/{listing_id}
        sale = self.soup.find(name='span', attrs={'class': 'pdp-history__details'})
        if sale and sale.text.strip() == 'This property is currently for sale':
            return sale.a['href'].split('/')[-1]
        else:
            return False

    def property_value(self):
        values = [currency_to_num(i.text) for i in
                  self.soup.find_all(name='p', attrs={'class': 'pdp-estimate__price'})]
        ranges = [currency_to_num(i) for i in
                  list(chain(*[i.text[7:].split(' - ')
                               for i in self.soup.find_all(name='p', attrs={'class': 'pdp-estimate__range'})]))]

        try:
            conf = float(self.soup.find(name='span',
                                        attrs={'class': 'pdp-confidence-rating__copy'}).text.strip().split('%')[0])
        except AttributeError:
            conf = np.nan

        if len(values) == 1:
            values.append(np.nan)
            ranges.extend([np.nan] * 2)
        elif not values:
            values.extend([np.nan] * 2)
            ranges.extend([np.nan] * 4)

        return {'buy': {'value': values[0], 'lower_bound': ranges[0], 'upper_bound': ranges[1]},
                'rent': {'value': values[1], 'lower_bound': ranges[2], 'upper_bound': ranges[3]},
                'confidence': conf}

    def value_change(self):
        period = [i.text for i in self.soup.find_all(name='span', attrs={'class': 'pdp-value-change__label'})]
        changes = [currency_to_num(i.text) for i in
                   self.soup.find_all(name='span', attrs={'class': 'pdp-value-change__value'})]
        diffs = [float(i.text.replace('%', '')) for i in
                 self.soup.find_all(name='span', attrs={'class': 'pdp-value-change__difference'})]

        df = pd.DataFrame(list(zip(period, changes, diffs)), columns=['period', 'value_change', 'perc_change'])
        df['value'] = self.property_value()['buy']['value'] / (1 + df.perc_change / 100)
        df['value'] = df.value.apply(myround)
        df = df[['period', 'value', 'value_change', 'perc_change']]

        return df

    def sales_history(self, dataframe=False):
        # historical listings available at https://www.zoopla.co.uk/property-history/{listing_id}
        history = {
            'date': [i.text for i in self.soup.find_all(name='span', attrs={'class': 'pdp-history__date'})],
            'status': [i.text for i in self.soup.find_all(name='span', attrs={'class': 'pdp-history__status'})],
            'price': [currency_to_num(i.text.replace('View listing', '')) for i in
                      self.soup.find_all(name='span', attrs={'class': 'pdp-history__price'})],
            'listing_id': []
        }

        for listing in self.soup.find_all(name='span', attrs={'class': 'pdp-history__price'}):
            try:
                history['listing_id'].append(listing.a['href'].split('/')[-1])
            except TypeError:
                history['listing_id'].append(np.nan)

        return pd.DataFrame(history) if dataframe else history

    def all_data(self, dataframe=True):

        data = self.details()
        data['address'] = str(self).split('Property details for ')[-1].split(' - Zoopla')[0]
        data['id'] = self.listing_id
        data['geolocation'] = self.location()
        data['for_sale_id'] = self.for_sale()
        data['property_value'] = self.property_value()
        data['value_change'] = self.value_change().to_dict()
        data['sales_history'] = self.sales_history()
        data['date_generated'] = datetime.utcnow()

        return pd.DataFrame.from_dict(data, orient='index').T if dataframe else data



from datetime import datetime
import pandas as pd

from pyzoopla.base import BaseProperty
from pyzoopla.utils import currency_to_num, text_inbetween, text_or_none


class PropertyListing(BaseProperty):
    """
    For sale listings available at https://ww2.zoopla.co.uk/for-sale/details/{listing_id}
    """

    def __init__(self, listing_id):
        super(PropertyListing, self).__init__(listing_id, slug='for-sale/details')

    def _price_history(self):
        dates = [i.text for i in self.soup.find_all(name='span', attrs={'class': 'dp-price-history__item-date'})]
        prices = [currency_to_num(i.text) for i in
                  self.soup.find_all(name='span', attrs={'class': 'dp-price-history__item-price'})]
        details = [i.text.strip() for i in
                   self.soup.find_all(name='span', attrs={'class': 'dp-price-history__item-detail'})]

        return {'date': dates, 'price': prices, 'detail': details}

    def details(self, dataframe=True):
        description = text_or_none(self.soup.find(name='div', attrs={'class': 'dp-description__text'}))
        main = [icon.text.strip() for icon in
                self.soup.find_all(name='li', attrs={'class': 'ui-list-icons__item'})]
        features = text_or_none(self.soup.find(name='ul',
                                               attrs={'class': 'dp-features__list ui-list-bullets'})).split('\n')[1:-1]
        price_history = self._price_history()
        data = {'listing_id': self.listing_id, 'description': description, 'main_features': main,
                'more_features': features, 'price_history': price_history, 'date_generated': datetime.utcnow()}

        return pd.DataFrame.from_dict(data, orient='index').T if dataframe else data


class PropertyHistoricalListing(BaseProperty):
    """
    Historical listings available at https://www.zoopla.co.uk/property-history/{listing_id}
    """

    def __init__(self, listing_id):
        super(PropertyHistoricalListing, self).__init__(listing_id, slug='property-history')

    def __str__(self):
        # Address of the property and listing date
        return self.soup.find(name='h1', attrs={'class': 'bottom-half'}).text.strip()

    def details(self, dataframe=True):
        string = self.soup.find(name='div', attrs={'id': 'historic-listing-content'}).text
        features = text_inbetween(string.replace('\n', '. '), 'Property features', 'Property description')
        description = text_inbetween(string.replace('\n', '. '), 'Property description', 'Previously marketed by')
        data = {'listing_id': self.listing_id, 'description': description, 'features': features,
                'date_generated': datetime.utcnow()}

        return pd.DataFrame.from_dict(data, orient='index').T if dataframe else data



html1 = """<li class="clearfix">
<span class="interface nearby_stations_schools_national_rail_station" title="London Bridge"></span>
<span class="nearby_stations_schools_name" title="London Bridge">London Bridge</span>


            (0.4 miles)

        </li>"""


html2 = """<li class="clearfix">
<span class="interface nearby_stations_schools_london_underground_station" title="Borough"></span>
<span class="nearby_stations_schools_name" title="Borough">Borough</span>


            (1.2 miles)

        </li>"""



import logging
import pymysql
import requests
import sys
import time

from pyzoopla.base_scraper import BasePurchaseScraper, BaseScraper
from pyzoopla.for_sale import ForSaleSearch
from pyzoopla.listing import PropertyListing
from pyzoopla.prices import PricesSearch
from pyzoopla.properties import PropertyDetails
from pyzoopla.to_rent import ToRentSearch
from pyzoopla.utils import insert_into_db

logging.basicConfig(level=logging.INFO, format='%(asctime)-15s %(funcName)s:%(lineno)d %(levelname)s - %(msg)s')


class PropertyScraper(BaseScraper):

    def search_prices(self):
        search = PricesSearch(location=self.location)

        try:
            _ = search.assumed_search_location
        except AttributeError:
            logging.critical('No location named \'{}\''.format(self.location))
            sys.exit()

        logging.info('Found {:,} properties in location: {}.'.format(search.total_properties,
                                                                     search.assumed_search_location))

        if search.total_properties > 100000:
            logging.warning('A maximum of 100,000 properties can be scraped for this location.')

        return search, search.assumed_search_location, search.total_pages

    def _get_property_ids(self, page_start=None, page_end=None):
        props, location, n_pages = self.search_prices()
        logging.info('About to scrape {} pages of properties for location {}.'.format(n_pages, location))
        return props.all_properties(page_start, page_end)

    def save_data(self, database=None, port=None, user=None, password=None):

        db_conn = pymysql.connect(host=database, port=port, user=user, password=password,
                                  cursorclass=pymysql.cursors.DictCursor)
        logging.info('Connected to database host: {}'.format(database))

        for prop_id in self._get_property_ids():

            try:
                logging.info('Scraping details for property ID: {}'.format(prop_id))
                prop = PropertyDetails(prop_id)
                insert_into_db(db_conn=db_conn, data=prop.all_data(dataframe=False), schema='zdb',
                               table='property_details')

                listing_id = prop.for_sale()
                if listing_id:
                    logging.info('Scraping details for property listing ID: {}'.format(listing_id))
                    listing = PropertyListing(listing_id)
                    insert_into_db(db_conn=db_conn, data=listing.details(dataframe=False), schema='zdb',
                                   table='listings_description')

            except AttributeError as err:
                logging.warning('Could not scrape property ID {}: {}'.format(prop_id, err))
                continue

            except requests.ConnectionError as err:
                logging.warning('Requests connection error occurred for property ID {}: {}'.format(prop_id, err))
                time.sleep(3)

        db_conn.close()
        logging.info('Finished scraping property details for location {}.'.format(self.location))


class ForSaleScraper(BasePurchaseScraper):
    def __init__(self, location, distance):
        super(ForSaleScraper, self).__init__(location, distance, ForSaleSearch)


class ToRentScraper(BasePurchaseScraper):
    def __init__(self, location, distance):
        super(ToRentScraper, self).__init__(location, distance, ToRentSearch)



from itertools import chain
import requests

from pyzoopla.base import BaseSearch
from pyzoopla.utils import headers


class PricesSearch(BaseSearch):
    """
    Get a list of property ID's from https://www.zoopla.co.uk/house-prices/{location}

    num_months: sales in the last {3, 6, 12, 60, 120, 240} months
    property_type_code: {D, S, T, F}
    """

    page_size = 40

    def __init__(self, location, num_months=None, property_type_code=None):
        self.num_months = num_months
        self.property_type_code = property_type_code
        super(PricesSearch, self).__init__(location, self.page_size)

    def _get_html(self, page_no=None):
        payload = {'num_months': self.num_months, 'property_type_code': self.property_type_code, 'pn': page_no}
        url = 'https://www.zoopla.co.uk/house-prices/{}/'.format(self.location)
        return requests.get(url, params=payload, headers=headers)

    @property
    def assumed_search_location(self):
        return self.soup.find(name='h1', attrs={'class': 'h-header'}).text.split('House prices in ')[-1]

    @staticmethod
    def _extract_ids(soup):
        return [address.a['href'].split('/')[-1] for address in
                soup.find_all(name='td', attrs={'class': 'browse-cell-address'})]

    def _get_ids_page(self, page_no):
        """
        Get list of property id's from a single page
        """
        soup = self._get_soup(page_no)
        return self._extract_ids(soup)

    def all_properties(self, page_start=None, page_end=None):
        """
        List of all property ID's for specified location.
        :param page_start: page no. to start scraping from
        :param page_end: page no. to scrape till
        :return: list of ID's
        """

        page_start = page_start if page_start else 1
        page_end = page_end if page_end else self.total_pages + 1

        if page_end == 1:
            return self._extract_ids(self.soup)
        else:
            prop_id_list = [self._get_ids_page(page_no) for page_no in range(page_start, page_end + 1)]
            return list(chain(*prop_id_list))

    def market_activity(self, period=20, property_type='all'):
        """
        Historical market activity for this location.

        :param period: last n years where n is measured in years,
                       options are {0.25, 0.5, 1, 5, 10, 20}
        :param property_type: options are {'all', 'detached', 'semi', 'terraced', 'flat'}
        :return: dictionary containing values for average price paid, no. of sales,
                 current average value and value change.
        """
        period_map = {0.25: 0, 0.5: 1, 1: 2, 5: 3, 10: 4, 20: 5}
        prop_type_map = {'all': 'data-value-all', 'detached': 'data-value-d', 'semi': 'data-value-s',
                         'terraced': 'data-value-t', 'flat': 'data-value-f'}

        if period not in period_map.keys():
            raise ValueError('period should be one of {}'.format(list(period_map.keys())))

        if property_type not in prop_type_map.keys():
            raise ValueError('property_type should be one of {}'.format(list(prop_type_map.keys())))

        summary = {'period': period, 'property_type': property_type}
        for stat in self.soup.find_all(name='span', attrs={'class': 'market-panel-stat-element-value'}):
            stat_name = stat['class'][1].split('stats-')[-1].replace('-', '_')
            summary[stat_name] = int(stat[prop_type_map[property_type]].split(',')[period_map[period]])

        return summary



# -*- coding: utf-8 -*-

"""
Utility functions that are commonly used within the package.
"""

from datetime import datetime
import os
import re


headers = {'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_13_6) AppleWebKit/537.36 (KHTML, like Gecko) '
                         'Chrome/68.0.3440.106 Safari/537.36'}


def currency_to_num(string, data_type=int):
    """
    Converts a pound sterling currency value into a number.
    >>> currency_to_num("£250,000")
    >>> 250000
    :param string: value of currency as a string
    :param data_type: intended data type of output
    :return: numerical value of currency
    """

    value = string.strip().replace('£', '').replace(',', '').replace('pcm', '')
    try:
        return data_type(value)
    except ValueError:
        return value


def myround(num, base=50):
    """
    Round a number to a nearest number specified by `base`.
    >>> myround(110380, base=50)
    >>> 110400
    :param num: number to round
    :param base: number to which to round to nearest
    :return: rounded number
    """
    return int(base * round(float(num)/base))


def text_or_none(soup, data_type=str):
    """Converts soup text to a data type else return None if soup has no text attribute."""
    try:
        return data_type(soup.text)
    except AttributeError:
        return ''


def to_datetime(date):
    """
    Converts string to a date type.
    >>> to_datetime('21st May 2012')
    >>> datetime(2012, 5, 21, 0, 0)
    """
    date = re.sub(r'\d+(st|nd|rd|th)', lambda m: m.group()[:-2].zfill(2), date)
    return datetime.strptime(date, '%d %b %Y')


def get_station_name(soup, station_num):
    """Extracts station name from string."""
    return soup[station_num].text.strip().split('(')[0].strip()


def dist_to_num(soup):
    """Extracts number from string e.g. '(0.4 miles)' -> 0.4"""
    return float(soup.text.strip().split('(')[-1].replace(')', '').split(' ')[0])


def text_inbetween(text, left, right):
    """Gets string in between two other strings, `left` and `right`."""
    return re.search(r'{}(.*){}'.format(left, right), text).group(1)


def output_data(df, location, output_dir='data'):
    """
    Outputs csv data to disk.  Checks to see if file exists first in which case
    it will append to this file otherwise create new file.
    """

    if not os.path.exists(output_dir):
        os.makedirs(output_dir)

    file_path = '{}/data_{}.csv'.format(output_dir, location).lower()
    if os.path.isfile(file_path):
        df.to_csv(file_path, mode='a', index=False, header=False)
    else:
        df.to_csv(file_path, index=False)


def insert_into_db(db_conn, data, schema, table):
    """
    Insert dictionary data into a sql database.
    :param db_conn: pymysql database connection
    :param data: dictionary values
    :param schema: name of sql schema
    :param table: name of sql table
    """
    placeholder = ', '.join(['%s'] * len(data))
    insert_query = 'replace into {schema}.{table} ({columns}) values ({values});'.format(
        schema=schema, table=table, columns=','.join(data.keys()), values=placeholder)
    cur = db_conn.cursor()
    cur.execute(insert_query, [str(i) for i in data.values()])
    db_conn.commit()
